Permissions

Struct Permissions 

pub struct Permissions {
    pub app: AppPermissions,
    pub key: KeyPermissions,
}
Global (PICC-wide) and Application-specific permissions and configuration.

Fields

app: AppPermissions

Permissions regarding what operations can be done in what state(s).

key: KeyPermissions

Permissions regarding key changes (specifically) within the current Application.

Implementations

Source

impl Permissions

pub const FACTORY_DEFAULT: Self

Default default application MIFARE DESFire EV3 key settings (0x0F)

Source

impl Permissions

pub fn from_u8(ks: u8) -> Self

Create a new Permissions from the provided Permissions encoded value.

pub fn as_u8<IoBackendErrorT>(&self) -> Result<u8, Error<IoBackendErrorT>>

Create a new Permissions encoded value.

‘IoBackendErrorT’ is generic here because Error requires it, this function does not conduct any I/O on its own. We will never return an Error::IoBackend, so this is purely for sizing/checking.