ApplicationId

Type Alias ApplicationId 

pub type ApplicationId = [u8; 3];
MiFare DESFire Cards can have a number of “applications”, identified by an ApplicationId. These IDs are unique within a Card (or I guess more specifically, a Card ecosystem), used to create a logical group of Files, Keys and access control to serve the needs of the system.